﻿#region License Information
/* HeuristicLab
 * Copyright (C) 2002-2014 Heuristic and Evolutionary Algorithms Laboratory (HEAL)
 *
 * This file is part of HeuristicLab.
 *
 * HeuristicLab is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * HeuristicLab is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with HeuristicLab. If not, see <http://www.gnu.org/licenses/>.
 */
#endregion

using System;
using System.Collections.Generic;
using System.Drawing;
using System.Linq;
using HeuristicLab.Common;
using HeuristicLab.Core;
using HeuristicLab.Persistence.Default.CompositeSerializers.Storable;

namespace HeuristicLab.EvolutionTracking {
  [Item("DirectedGraph", "Generic class representing a directed graph with custom vertices and content")]
  [StorableClass]
  public class DirectedGraph : Item, IDirectedGraph {
    [Storable]
    protected readonly List<IVertex> nodes; // for performance reasons, maybe this should be a linked list (fast remove)
    public List<IVertex> Nodes {
      get { return nodes; }
    }
    [Storable]
    private readonly Dictionary<object, List<IVertex>> contentMap;
    public DirectedGraph() {
      nodes = new List<IVertex>();
      contentMap = new Dictionary<object, List<IVertex>>();
    }
    [StorableConstructor]
    protected DirectedGraph(bool serializing)
      : base(serializing) {
    }
    protected DirectedGraph(DirectedGraph original, Cloner cloner)
      : base(original, cloner) {
      nodes = new List<IVertex>(original.Nodes);
      contentMap = new Dictionary<object, List<IVertex>>(original.contentMap);
    }
    public override IDeepCloneable Clone(Cloner cloner) {
      return new DirectedGraph(this, cloner);
    }
    public bool Contains(IVertex t) {
      return nodes.Contains(t);
    }

    public bool Contains(object content) {
      return contentMap.ContainsKey(content);
    }
    public List<IVertex> this[object key] {
      get {
        List<IVertex> result;
        contentMap.TryGetValue(key, out result);
        return result;
      }
    }
    public virtual bool Any(Func<IVertex, bool> predicate) {
      return nodes.Any(predicate);
    }
    public virtual bool IsEmpty {
      get { return !nodes.Any(); }
    }
    public virtual void Clear() {
      nodes.Clear();
    }
    public virtual void AddVertex(IVertex vertex) {
      Nodes.Add(vertex);
      if (!contentMap.ContainsKey(vertex.Content))
        contentMap[vertex.Content] = new List<IVertex>();
      contentMap[vertex.Content].Add(vertex);
    }

    public virtual void RemoveVertex(IVertex vertex) {
      nodes.Remove(vertex);
    }
    public override Image ItemImage {
      get { return Common.Resources.VSImageLibrary.Graph; }
    }
  }
}
